/**
 * Copyright © 2014-2023 PDF Technologies, Inc. All Rights Reserved.
 * <p>
 * THIS SOURCE CODE AND ANY ACCOMPANYING DOCUMENTATION ARE PROTECTED BY INTERNATIONAL COPYRIGHT LAW
 * AND MAY NOT BE RESOLD OR REDISTRIBUTED. USAGE IS BOUND TO THE ComPDFKit LICENSE AGREEMENT.
 * UNAUTHORIZED REPRODUCTION OR DISTRIBUTION IS SUBJECT TO CIVIL AND CRIMINAL PENALTIES.
 * This notice may not be removed from this file.
 */

package com.compdfkit.tools.common.views.pdfproperties.font;

import android.content.Context;
import android.graphics.Typeface;
import android.util.AttributeSet;
import android.view.View;
import android.widget.AdapterView;
import android.widget.LinearLayout;
import android.widget.Spinner;

import androidx.annotation.Nullable;

import com.compdfkit.core.annotation.CPDFTextAttribute;
import com.compdfkit.core.font.CPDFFont;
import com.compdfkit.core.font.CPDFFontName;
import com.compdfkit.tools.R;
import com.compdfkit.tools.common.utils.CLog;

import java.util.ArrayList;


public class CPDFFontView extends LinearLayout {

    private Spinner fontSpinner;

    private Spinner fontStyleSpinner;

    private CFontSpinnerAdapter1 fontAdapter;

    private CFontStyleSpinnerAdapter fontStyleSpinnerAdapter;

    private boolean fontIsBold = false;

    private boolean fontIsItalic = false;

    private CFontChangeListener fontChangeListener;

    private CFontTypefaceListener typefaceListener;

    public CPDFFontView(Context context) {
        this(context, null);
    }

    public CPDFFontView(Context context, @Nullable AttributeSet attrs) {
        this(context, attrs, 0);
    }

    public CPDFFontView(Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        initView(context);
    }

    private void initView(Context context) {
        View view = inflate(context, R.layout.tools_cpdf_font_view, this);
        fontSpinner = view.findViewById(R.id.spinner_font);
        fontStyleSpinner = view.findViewById(R.id.spinner_font_style);
        if (!isInEditMode()) {
            fontAdapter = new CFontSpinnerAdapter1(context);
            fontSpinner.setAdapter(fontAdapter);


            fontStyleSpinnerAdapter = new CFontStyleSpinnerAdapter(context);
            fontStyleSpinner.setAdapter(fontStyleSpinnerAdapter);

        }
    }

    private int fontCheck = 0;
    private int fontStyleCheck = 0;

    public void initFont(String fontName, boolean isBold, boolean isItalic) {
        fontCheck = 0;
        fontStyleCheck = 0;
        fontSpinner.setSelection(fontAdapter.getPosition(fontName));
        switch (fontName) {
            case "Courier":
            case "Helvetica":
            case "Times_Roman":
                fontStyleSpinnerAdapter.updateFontStyle(CPDFTextAttribute.FontNameHelper.FontType.valueOf(fontName));
                break;
            default:
                CPDFFontName cpdfFontName = fontAdapter.getCPDFFontName(fontName);
                if (cpdfFontName != null) {
                    fontStyleSpinnerAdapter.updateFontStyle(cpdfFontName);
                } else {
                    fontStyleSpinnerAdapter.updateFontStyle(CPDFTextAttribute.FontNameHelper.FontType.Helvetica);
                }
                break;
        }
        int fontStyleIndex = fontStyleSpinnerAdapter.getIndex(isBold, isItalic);
        fontStyleSpinner.setSelection(fontStyleIndex);
        updateStyle(fontStyleSpinnerAdapter.getItem(fontStyleIndex));

        fontSpinner.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                if (++fontCheck > 1) {

                    Object fontObj = fontAdapter.getItem(position);
                    if (fontObj instanceof CPDFTextAttribute.FontNameHelper.FontType) {
                        if (fontChangeListener != null) {
                            fontChangeListener.font((CPDFTextAttribute.FontNameHelper.FontType) fontObj, null);
                        }
                        fontStyleSpinnerAdapter.updateFontStyle((CPDFTextAttribute.FontNameHelper.FontType) fontObj);
                        resetFontStyle();
                    } else if (fontObj instanceof CPDFFontName) {
                        CPDFFontName cpdfFontName = (CPDFFontName) fontObj;
                        if (fontChangeListener != null) {
                            fontChangeListener.font(null, cpdfFontName.getFamilyName());
                        }
                        ArrayList<String> psNames = cpdfFontName.getPsName();
                        CLog.e("字体", cpdfFontName.getFamilyName() + " styleName: " + cpdfFontName.getStyleName());
                        CLog.e("字体", "psName----- >");
                        for (String psName : psNames) {
                            CLog.e("字体", psName);
                        }
                        CLog.e("字体", "psName----- <");
                        fontStyleSpinnerAdapter.updateFontStyle(((CPDFFontName) fontObj));
                        resetFontStyle();
                    }
                    changeTypefaceListener();
                }
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {

            }
        });


        fontStyleSpinner.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                if (++fontStyleCheck > 1){
                    CFontStyleSpinnerAdapter.FontStyle fontStyle = fontStyleSpinnerAdapter.getItem(position);
                    updateStyle(fontStyle);
                    if (fontChangeListener != null) {
                        fontChangeListener.fontStyle(fontIsBold, fontIsItalic);
                    }
                    changeTypefaceListener();
                }
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {

            }
        });

    }

    private void changeTypefaceListener() {
        if (typefaceListener != null) {
            Object font = fontSpinner.getSelectedItem();
            if (font instanceof CPDFTextAttribute.FontNameHelper.FontType) {
                String fontTypeName = CPDFTextAttribute.FontNameHelper.obtainFontName((CPDFTextAttribute.FontNameHelper.FontType) font, fontIsBold, fontIsItalic);
                Typeface typeface = CPDFTextAttribute.FontNameHelper.getTypeface(getContext(), fontTypeName);
                typefaceListener.typeface(typeface);
            } else if (font instanceof CPDFFontName) {
                String psName = CPDFFont.getPsName(((CPDFFontName) font).getFamilyName(), fontIsBold, fontIsItalic);
                Typeface typeface = CPDFTextAttribute.FontNameHelper.getOutTypeface(getContext(), psName);
                typefaceListener.typeface(typeface);
            }
        }
    }

    private void resetFontStyle() {
        fontIsBold = false;
        fontIsItalic = false;
        fontStyleSpinner.setSelection(0);
    }

    private void updateStyle(CFontStyleSpinnerAdapter.FontStyle fontStyle) {
        switch (fontStyle) {
            case Bold:
                fontIsBold = true;
                fontIsItalic = false;
                break;
            case Italic:
                fontIsBold = false;
                fontIsItalic = true;
                break;
            case Bold_Italic:
                fontIsBold = true;
                fontIsItalic = true;
                break;
            default:
                fontIsBold = false;
                fontIsItalic = false;
                break;
        }
    }

    public void setFontChangeListener(CFontChangeListener fontChangeListener) {
        this.fontChangeListener = fontChangeListener;
    }

    public void setTypefaceListener(CFontTypefaceListener typefaceListener) {
        this.typefaceListener = typefaceListener;
    }

    public interface CFontChangeListener {
        void font(CPDFTextAttribute.FontNameHelper.FontType fontType, String outFontName);


        void fontStyle(boolean bold, boolean italic);
    }


    public interface CFontTypefaceListener {
        void typeface(Typeface typeface);
    }

}
